package janvlk.odhadceny.main.engine;

import janvlk.odhadceny.main.entities.*;
import janvlk.odhadceny.main.data.*;
import janvlk.odhadceny.main.ui.MainWindow;

import javax.swing.*;

/**
 * Trida poskytujici metody pro vypocet odhadu ceny pro zadane parametry.
 * <p>
 * Created by Jan Vlk on 28.02.2016.
 */
public class PriceCalculator {

    private Data data;

    /**
     * Metoda vypocitavajici odhad ceny na zaklade zadanych parametru.
     *
     * @param year       Rok vyroby.
     * @param volume     Objem motoru.
     * @param equipment  Druh vybavy.
     * @param km         Pocet tisicu najetych kilometru.
     * @param fuel       Druh pohonu.
     * @param chassi     Druh sasi.
     * @param gear       Druh prevodovky.
     * @param service    Druh servisu.
     * @param xenon      Druh svetlometu.
     * @param navigation Druh navigacniho systemu.
     * @param avd        Druh pohonu napravy.
     * @param rs         Druh sportovni rady.
     * @param scout      Druh terenni rady.
     * @return Odhadnuta cena.
     */
    public long calculatePrice(int year, int volume, Equipment equipment, int km, Fuel fuel, Chassi chassi, Gear gear, Service service, Xenon xenon, Navigation navigation, Avd avd, Rs rs, Scout scout) {

        //vyber zdroje dat pro dany rok
        chooseDataSource(year);

        //vypocet ceny
        double price = data.getCONST() +
                (volume * data.getVOLUME()) +
                (equipment.getValue() * data.getEQUIPMENT()) +
                (km * data.getKM()) +
                (fuel.getValue() * data.getFUEL()) +
                (chassi.getValue() * data.getCHASSI()) +
                (gear.getValue() * data.getGEAR()) +
                (service.getValue() * data.getSERVICE()) +
                (xenon.getValue() * data.getXENON()) +
                (navigation.getValue() * data.getNAVIGATION()) +
                (avd.getValue() * data.getAVD()) +
                (rs.getValue() * data.getRS()) +
                (scout.getValue() * data.getSCOUT());

        long roundedPrice = Math.round(price * 1000);

        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("Rok: " + year + " [" + data.getCONST() + "]" + System.lineSeparator());
        stringBuilder.append("Objem: " + volume + " [" + data.getVOLUME() + "]" + System.lineSeparator());
        stringBuilder.append("Vybava: " + equipment.toString() + " (" + equipment.getValue() + ")" + " [" + data.getEQUIPMENT() + "]" + System.lineSeparator());
        stringBuilder.append("Km: " + km + " [" + data.getKM() + "]" + System.lineSeparator());
        stringBuilder.append("Palivo: " + fuel.name() + " (" + fuel.getValue() + ")" + " [" + data.getFUEL() + "]" + System.lineSeparator());
        stringBuilder.append("Sasi: " + chassi.name() + " (" + chassi.getValue() + ")" + " [" + data.getCHASSI() + "]" + System.lineSeparator());
        stringBuilder.append("Prevodovka: " + gear.name() + " (" + gear.getValue() + ")" + " [" + data.getGEAR() + "]" + System.lineSeparator());
        stringBuilder.append("Servis: " + service.name() + " (" + service.getValue() + ")" + " [" + data.getSERVICE() + "]" + System.lineSeparator());
        stringBuilder.append("Xenony: " + xenon.name() + " (" + xenon.getValue() + ")" + " [" + data.getXENON() + "]" + System.lineSeparator());
        stringBuilder.append("Navigace: " + navigation.name() + " (" + navigation.getValue() + ")" + " [" + data.getNAVIGATION() + "]" + System.lineSeparator());
        stringBuilder.append("Ctyrkolka: " + avd.name() + " (" + avd.getValue() + ")" + " [" + data.getAVD() + "]" + System.lineSeparator());
        stringBuilder.append("RS: " + rs.name() + " (" + rs.getValue() + ")" + " [" + data.getRS() + "]" + System.lineSeparator());
        stringBuilder.append("Scout: " + scout.name() + " (" + scout.getValue() + ")" + " [" + data.getSCOUT() + "]" + System.lineSeparator());
        stringBuilder.append("Odhadnuta cena: " + roundedPrice + " CZK" + System.lineSeparator());

        /* nasledujici radek lze odkomentovat pro kontrolu vstupnich dat */
        //JOptionPane.showMessageDialog(MainWindow.frame, stringBuilder.toString(), "Zadane hodnoty", JOptionPane.PLAIN_MESSAGE);

        return roundedPrice;
    }

    /**
     * Vybere zdroj dat pro dany rok.
     *
     * @param year Rok, pro ktery se maji nastavit konstanty.
     */
    private void chooseDataSource(int year) {
        switch (year) {
            case 2005: {
                data = new Data2005();
                break;
            }
            case 2006: {
                data = new Data2006();
                break;
            }
            case 2007: {
                data = new Data2007();
                break;
            }
            case 2008: {
                data = new Data2008();
                break;
            }
            case 2009: {
                data = new Data2009();
                break;
            }
            case 2010: {
                data = new Data2010();
                break;
            }
            case 2011: {
                data = new Data2011();
                break;
            }
            case 2012: {
                data = new Data2012();
                break;
            }
        }
    }
}
